let oauth2Server = require('oauth2-server');
let AuthService = require('../services/auth');
let responseService = require('../services/response');
let jwt = require('jsonwebtoken');

function AuthController(app) {

    let Request = oauth2Server.Request;
    let Response = oauth2Server.Response;
    let authService = new AuthService(app);

    /**
     * Validates user and sends a OAuth2 token
     * @param {Request} req Node request
     * @param {Response} res Node resposne
     */
    this.loginWithOAuth2 = async (req, res) => {
        let request = getLoginRequest(req);
        let response = new Response(res);
        try {
            await app.oauth.token(request, response);
            let token = jwt.sign({ user: req.body.username }, authService.getSecret());
            res.json({ token: token });
        } catch (err) {
            responseService.error(res, err.status, err.code, err.message);
        }
    };

    /**
     * Validates user and send a json web token
     * @param {Request} req Node request
     * @param {Response} res Node response
     */
    this.loginWithJWT = async (req, res) => {
        if (authService.authUserJWT(req.body.username, req.body.password)) {
            let user = await authService.getRandomUser();
            let token = jwt.sign({ email: user.email, id: user.login.uuid }, authService.getSecret());
            authService.setLoginAsJWT();
            res.json({ data: Object.assign(user, { token }) });
        } else {
            responseService.throwNotFound(res);
        }
    };

    /**
     * Retrieves an OAuth2 request with specified client
     * @param {Request} req Node request
     * @returns {Request} OAuth2 request
     */
    let getLoginRequest = (req) => {
        req.body.username = 'zeedhi@teknisa.com';
        req.body.password = 'zeedhi';
        return new Request(req);
    };

    /**
     * OLD LOGIN ROUTES
     */
    this.productLanguages = async (req, res) => {
        res.json({
            dataset: {
                languages: {
                    DEFAULT_LANGUAGE: {},
                    LANGUAGES: [
                        { NAME: 'Português (Brasil)', SHORT_NAME: 'pt_br' },
                        { NAME: 'English (USA)', SHORT_NAME: 'en_us' },
                        { NAME: 'Español (ESP)', SHORT_NAME: 'es_es' },
                    ],
                },
            },
        });
    } 

    this.setLogAccess = async (req, res) => {
        res.json({});
    } 

    this.updatePassword = async (req, res) => {
        res.json({});
    } 

    this.requestNewPassword = async (req, res) => {
        res.json({});
    } 

    this.testLicenseConnection = async (req, res) => {
        res.json({
            dataset: {
                testLicenseConnection: {
                    ERROR: false,
                    URL: 'https://license.teknisa.com',
                },
            },
        });
    } 

    this.findPrivacyPolicyByAuthentication = async (req, res) => {
        const filter = {};
        req.body.filter.forEach((item) => {
            filter[item.name] = item.value;
        });
        res.json({
            dataset: {
                findPrivacyPolicyByAuthentication: [
                    {
                        path: 'https://security.teknisa.com/_pdf/Politica_LGPD_Padrao.pdf',
                        status: filter.USER === '1',
                    },
                ],
            },
        });
    } 

    this.login = async (req, res) => {
        const filter = {};
        req.body.filter.forEach((item) => {
            filter[item.name] = item.value;
        });

        // password will expire in 2 days
        if (filter.EMAIL === '1') {
            res.json({
                dataset: {
                    userData: {
                        USER: '001',
                        USER_NAME: 'Usuário 1',
                        TOKEN: 'poid;lkfqp394782-0eoif',
                        IS_SUPPORT_OPERATOR: false,
                        LOGGED: true,
                        EXPIRED_PASSWORD: 'PASSWORD_WILL_EXPIRE_2',
                    },
                },
            });
        } else if (filter.EMAIL === '2') { // password will expire today
            res.json({
                dataset: {
                    userData: {
                        USER: '002',
                        USER_NAME: 'Usuário 2',
                        TOKEN: 'poid;lkfqp394782-0eoif',
                        IS_SUPPORT_OPERATOR: false,
                        LOGGED: true,
                        EXPIRED_PASSWORD: 'PASSWORD_EXPIRE_TODAY',
                    },
                },
            });
        } else if (filter.EMAIL === '3') { // password expired
            res.json({
                dataset: {
                    userData: {
                        USER: '003',
                        USER_NAME: 'Usuário 3',
                        TOKEN: 'poid;lkfqp394782-0eoif',
                        IS_SUPPORT_OPERATOR: false,
                        LOGGED: true,
                        EXPIRED_PASSWORD: 'EXPIRED_PASSWORD',
                    },
                },
            });
        } else if (filter.EMAIL === '4') { // PER expired
            res.json({
                dataset: {
                    userData: {
                        USER: '003',
                        USER_NAME: 'Usuário 3',
                        TOKEN: 'poid;lkfqp394782-0eoif',
                        IS_SUPPORT_OPERATOR: false,
                        LOGGED: true,
                        ALERT_EXPIRATION_DATE_PER: 2,
                    },
                },
            });
        } else if (filter.EMAIL === '5') { // invalid password
            res.json({
                error: '{"CODE":"INVALID_PASSWORD","PARAMS":[]}',
                errorCode: 32,
                exceptions: [],
            });
        } else if (filter.EMAIL === '6') { // concurrent access
            if (!filter.SESSION_CHANGE) {
                res.json({
                    error: '{"CODE":"ALE_CONCURRENT_ACCESS","PARAMS":[]}',
                    errorCode: 35,
                    exceptions: [],
                });
            } else {
                res.json({
                    dataset: {
                        userData: {
                            USER: '006',
                            USER_NAME: 'Usuário 6',
                            TOKEN: 'poid;lkfqp394782-0eoif',
                            IS_SUPPORT_OPERATOR: false,
                            LOGGED: true,
                        },
                    },
                });
            }
        } else { // OK
            res.json({
                dataset: {
                    userData: {
                        USER: '999',
                        USER_NAME: 'Admin',
                        TOKEN: 'poid;lkfqp394782-0eoif',
                        IS_SUPPORT_OPERATOR: false,
                        LOGGED: true,
                    },
                },
            });
        }
    } 
}

module = module.exports = AuthController;
