let fs = require('fs');
let responseService = require('../services/response');

function MetadataController() {
    let viewsPath = __dirname + '/../metadata/views/';
    let staticDataPath = __dirname + '/../metadata/staticdata/';

    /**
     * Send a component metadata to client
     * @param {Request} req Node request
     * @param {Response} res Node response
     */
    this.getMetadata = (req, res) => {
        try {
            let component;
            let metadata = req.params.metadata;
            if (fs.existsSync(getViewPath(metadata))) {
                component = getComponent(metadata);
            } else if (fs.existsSync(getDataPath(metadata))) {
                component = getStaticDataContent(metadata);
            } else {
                throw 'Not Found';
            }
            res.json(component);
        } catch (err) {
            responseService.throwNotFound(res);
        }
    };

    /**
     * Send a menu metadata to client
     * @param {Request} req Node request
     * @param {Response} res Node response
     */
    this.getMenu = (req, res) => {
        try {
            res.json(getMenuFileContent());
        } catch (err) {
            responseService.throwNotFound(res);
        }
    };

    /**
     * Retrieves a component metadata content
     * @param {String} componentName Component filename
     * @returns {*} Component content
     * @throws Will throw an error if component doesn't exists
     */
    let getComponent = (componentName) => {
        return JSON.parse(fs.readFileSync(getViewPath(componentName), 'utf8'));
    }

    let getStaticDataContent = (dataName) => {
        return JSON.parse(fs.readFileSync(getDataPath(dataName), 'utf8'));
    }

    let getMenuFileContent = () => {
        return JSON.parse(fs.readFileSync(getDataPath('menu'), 'utf8'));
    }

    let getViewPath = (componentName) => {
        return viewsPath + componentName + '.json';
    }

    let getDataPath = (dataName) => {
        return staticDataPath + dataName + '.json';
    }
}

module = module.exports = MetadataController;
